<?php

namespace Drupal\moderation_team\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\moderation_team\Event\NodeEvent;

/**
 * NodeSubscriber.
 */
class NodeSubscriber implements EventSubscriberInterface {

  /**
   * Add entity in queue.
   *
   * @param \Drupal\moderation_team\Event\NodeEvent $event
   */
  public function onNodeInsert(NodeEvent $event) {
    $entity = $event->getEntity();
    $database = \Drupal::database();

    $config = \Drupal::config('moderation_team.settings');
    $mt_content_type = $config->get('mt_content_type');
    $mt_queue_type = $config->get('mt_queue_type');
    $mt_max_queue = $config->get('mt_max_queue');

    // Check if content type match with configuration
    if($entity->getEntityTypeId() === 'node' &&  $entity->bundle() == $mt_content_type){

      // Check if queue type is automatic or manual
      $nid = $entity->get('nid')->value;

      if ($mt_queue_type == 'queue') {
        $max_queue = $config->get('mt_max_queue');

        // Get current queue
        $queue_service = \Drupal::service('moderation_team.queue');
        $queue = $queue_service->getCurrentQueue($max_queue);

        // Insert node into queue
        $result = $database->insert('mt_queue')
          ->fields([
            'queue_id' => $queue,
            'nid' => $nid,
            'created' => time(),
          ])
          ->execute();
      }
      else if ($mt_queue_type = 'automatic') {
        // Get all role based active users
        // $assinee = $this->getCurrentAssignee();
        $queue_service = \Drupal::service('moderation_team.queue');
        $assinee = $queue_service->getCurrentAssignee();

        if (!empty($assinee)) {
          $result = $database->insert('mt_node_queue')
            ->fields([
              'nid' => $nid,
              'reviewer_id' => $assinee,
              'created' => time(),
            ])
            ->execute();
        }
      }
    }
  }


  /**
   * Add entity in queue.
   *
   * @param \Drupal\moderation_team\Event\NodeEvent $event
   */
  public function onNodeDelete(NodeEvent $event) {
    $entity = $event->getEntity();
    $database = \Drupal::database();

    $config = \Drupal::config('moderation_team.settings');
    $mt_content_type = $config->get('mt_content_type');
    $mt_queue_type = $config->get('mt_queue_type');
    $mt_max_queue = $config->get('mt_max_queue');

    // Check if content type match with configuration
    if($entity->getEntityTypeId() === 'node' &&  $entity->bundle() == $mt_content_type) {

      // Check if queue type is automatic or manual
      $nid = $entity->get('nid')->value;

      if ($mt_queue_type == 'queue') {
        // Delete queue node
        $database->delete('mt_queue')
        ->condition('nid', $nid)
        ->execute();
      }
      else if ($mt_queue_type = 'automatic') {

        // Delete node queue
        $database->delete('mt_node_queue')
        ->condition('nid', $nid)
        ->execute();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[NodeEvent::NODE_INSERT][] = ['onNodeInsert'];
    $events[NodeEvent::NODE_DELETE][] = ['onNodeDelete'];
    return $events;
  }
}