<?php

namespace Drupal\moderation_team\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\moderation_team\Event\UserEvent;
use Drupal\moderation_team\Event\NodeEvent;

/**
 * Logs the creation of a new node.
 */
class UserSubscriber implements EventSubscriberInterface {

  /**
   * Update Assgnee of the nodes if assignee removed.
   *
   * @param \Drupal\moderation_team\Event\UserEvent $event
   */
  public function onUserUpdate(UserEvent $event) {

    $entity = $event->getEntity();
    $orgEntity = $entity->original;

    // Get id
    $uid = $entity->id();
    $arrRoles = $entity->getRoles();
    $arrOrgRoles = $orgEntity->getRoles();

    $config = \Drupal::config('moderation_team.settings');
    $queueType = $config->get('mt_queue_type');
    $role = $config->get('mt_role');

    // Check Queue type and user role
    if ($queueType == 'automatic' && in_array($role, $arrOrgRoles)) {

      $statusCondition = ($orgEntity->status->value == 1 && $entity->status->value == 0);
      $roleCondition = (!in_array($role, $arrRoles));

      // Update if user is blocked
      if ($statusCondition || $roleCondition) {

        // Get assignee node list of the user
        $queueService = \Drupal::service('moderation_team.queue');
        $queueService->reArrangeAssigneeNodes($uid);
      }
    }
  }

  /**
   * Update Assgnee of the nodes if assignee removed.
   *
   * @param \Drupal\moderation_team\Event\UserEvent $event
   */
  public function onUserDelete(UserEvent $event) {

    $entity = $event->getEntity();

    // Get id
    $uid = $entity->id();
    $arrRoles = $entity->getRoles();

    $config = \Drupal::config('moderation_team.settings');
    $queueType = $config->get('mt_queue_type');
    $role = $config->get('mt_role');

    // Check Queue type and user role
    if ($queueType == 'automatic' && in_array($role, $arrRoles)) {

      // Get assignee node list of the user
      $queueService = \Drupal::service('moderation_team.queue');
      $queueService->reArrangeAssigneeNodes($uid);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[UserEvent::USER_UPDATE][] = ['onUserUpdate'];
    $events[UserEvent::USER_DELETE][] = ['onUserDelete'];
    return $events;
  }
}