<?php

/**
 *
 */
namespace Drupal\moderation_team\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;


/**
 * Configure moderation_team settings for this site.
 */
class ModerationTeamSettingsForm extends ConfigFormBase {

  /** @var string Config settings */
  const SETTINGS = 'moderation_team.settings';

  /** 
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mt_admin_settings';
  }

  /** 
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /** 
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $types = [];
    $config = $this->config(static::SETTINGS);
    $node_types = \Drupal\node\Entity\NodeType::loadMultiple();

    foreach ($node_types as $node_type) {
      $types[$node_type->id()] = $node_type->label();
    }

    // Get all roles
    $roles = Role::loadMultiple();

    foreach ($roles as $key => $role) {
      $arr_roles[$role->id()] = $role->label();
    }

    $queue_types = ['automatic' => 'Automatic User Assignment', 'queue' => 'Manual Ad Hoc (Queue)'];

    $type_value= !empty($config->get('mt_queue_type')) ? $config->get('mt_queue_type') : 'automatic';
    $sync_value= !empty($config->get('mt_sync_data')) ? $config->get('mt_sync_data') : FALSE;
    $selected_role= !empty($config->get('mt_role')) ? $config->get('mt_role') : '';

    $form['mt_queue_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Assignment Type'),
      '#options' => $queue_types,
      '#default_value' => $type_value,
    ];

    $form['mt_content_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Content Type'),
      '#options' => $types,
      '#default_value' => $config->get('mt_content_type'),
    ];

    $form['mt_sync_data'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Sync existing content'),
      '#default_value' => $sync_value,
      // '#states' => array(
      //   // Only show this field when the value of type is automatic.
      //   'visible' => array(
      //     ':input[name="mt_queue_type"]' => array('value' => 'automatic'),
      //   ),
      // ),
    ];

    $form['mt_role'] = [
      '#type' => 'radios',
      '#title' => $this->t('Roles'),
      '#options' => $arr_roles,
      '#default_value' => $selected_role,
    ];

    $form['mt_max_queue'] = [
      '#type' => 'number',
      '#title' => $this->t('Max Queue'),
      '#default_value' => $config->get('mt_max_queue'),
      '#states' => array(
        // Only show this field when the value of type is queue.
        'visible' => array(
          ':input[name="mt_queue_type"]' => array('value' => 'queue'),
        ),
      ),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $queue_type = $form_state->getValue('mt_queue_type');
    $content_type = $form_state->getValue('mt_content_type');
    $max_queue = $form_state->getValue('mt_max_queue');
    $sync_data = $form_state->getValue('mt_sync_data');

    // Retrieve the configuration
    $this->configFactory->getEditable(static::SETTINGS)
    // Set the submitted configuration setting
    ->set('mt_queue_type', $queue_type)
    ->set('mt_content_type', $content_type)
    ->set('mt_sync_data', $sync_data)
    ->set('mt_role', $form_state->getValue('mt_role'))
    ->set('mt_max_queue', $max_queue)
    ->save();

    // @todo - add logic to sync existing data
    if ($sync_data) {
      $this->setAssignee($queue_type, $content_type, $max_queue);
    }

    parent::submitForm($form, $form_state);
  }

  public function setAssignee($queue_type, $content_type, $max_queue) {

    $queue_service = \Drupal::service('moderation_team.queue');

    // @todo - set automatic permission to selected role to access page
    // @todo - make const variable for queue type
    if ($queue_type == 'automatic') {

      // Create a subquery, to get existin queue nodes.
      $queue_nids = db_select('mt_node_queue', 'nq');
      $queue_nids->addField('nq', 'nid');

      // Get all pending nodes
      $db = \Drupal::database();
      $nodes = $db->select('node_field_data', 'n')
        ->fields('n', ['nid'])
        ->condition('n.type', $content_type)
        ->condition('n.nid', $queue_nids, 'NOT IN')
        ->execute()
        ->fetchAll();
      $arr_nodes = [];
      foreach ($nodes as $key => $node) {
        $arr_nodes[] = $node->nid;
      }

      foreach ($arr_nodes as $key => $nid) {
        // $assinee = \Drupal\moderation_team\EventSubscriber\NodeInsertSubscriber::getCurrentAssignee();
        $assinee = $queue_service->getCurrentAssignee();
        $result = $db->insert('mt_node_queue')
          ->fields([
            'nid' => $nid,
            'reviewer_id' => $assinee,
            'created' => time(),
          ])
          ->execute();
      }
    }
    elseif ($queue_type == 'queue') {
      // Create a subquery, to get existin queue nodes.
      $queue_nids = db_select('mt_queue', 'mq');
      $queue_nids->addField('mq', 'nid');

      // Get all pending nodes
      $db = \Drupal::database();
      $nodes = $db->select('node_field_data', 'n')
        ->fields('n', ['nid'])
        ->condition('n.type', $content_type)
        ->condition('n.nid', $queue_nids, 'NOT IN')
        ->execute()
        ->fetchAll();
      $arr_nodes = [];
      foreach ($nodes as $key => $node) {
        $arr_nodes[] = $node->nid;
      }

      foreach ($arr_nodes as $key => $nid) {
        // $queue = \Drupal\moderation_team\EventSubscriber\NodeInsertSubscriber::getCurrentQueue($max_queue);
        $queue = $queue_service->getCurrentQueue($max_queue);
        $result = $db->insert('mt_queue')
            ->fields([
              'queue_id' => $queue,
              'nid' => $nid,
              'created' => time(),
            ])
            ->execute();
      }
    }
  }
}
