<?php

/**
 * Class QueueService
 */
namespace Drupal\moderation_team;

use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\moderation_team\Event\NodeEvent;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\Role;

/**
 * QueueService service class.
 */
class QueueService {

  private $database;

  /**
   * construct function of the class
   */
  public function __construct(connection $database) {
    $this->database = $database;
  }

  /**
   * Get current queue count
   */
  public function getCurrentQueue($max_queue) {

    // Get latest queue id
    $queue_id = $this->database->select('mt_queue', 'mq')
      ->fields('mq', ['queue_id'])
      ->orderBy('id', 'DESC')
      ->execute()
      ->fetchField();

    $queue = ($queue_id == $max_queue) ? 1 : $queue_id + 1;
    return $queue;
  }

  /**
   * Get current assignee
   */
  public function getCurrentAssignee() {

    // Get all assignee
    $allAssignee = $this->getAllAssignee();

    // Get last assignee
    $lastAssignee = $this->getLastAssignee();

    // Search last assignee
    $search = array_search($lastAssignee, $allAssignee);

    $assignee = ($search !== false) ? isset($allAssignee[$search+1]) ? $allAssignee[$search+1] : array_shift($allAssignee) : array_shift($allAssignee);

    return $assignee;
  }

  /**
   * Get last assignee
   */
  public function getLastAssignee() {
    $assignee = $this->database->select('mt_node_queue', 'nq')
      ->fields('nq', ['reviewer_id'])
      ->orderBy('id', 'DESC')
      ->execute()
      ->fetchField();

    return $assignee;
  }

  /**
   * Get all Assignee
   */
  public function getAllAssignee() {
    $config = \Drupal::config('moderation_team.settings');
    $mt_role = $config->get('mt_role');

    $ids = \Drupal::entityQuery('user')
      ->condition('status', 1)
      ->condition('roles', $mt_role)
      ->execute();

    $arr_ids = array_values($ids);
    return $arr_ids;
  }

  /**
   * To remove asignment of provided user
   *
   * @param: uid: id of the assignee
   */
  function reArrangeAssigneeNodes($uid) {
    $nids = $this->getAssigneeNodes($uid);

    $nodes = [];
    if (!empty($nids)) {
      $nodes =  Node::loadMultiple($nids);
    }

    // Assign nodes to existing assignee
    foreach ($nodes as $key => $node) {
      \Drupal::service('event_dispatcher')->dispatch(NodeEvent::NODE_INSERT, new NodeEvent($node));
    }

    // Remove existing node for current user
    $this->removeAssigneeNodes($uid);
  }

  /**
   * To get node ids based on assigneee
   *
   * @param: reviewer_id: id of the assignee
   *
   * @return: array of node ids
   */
  function getAssigneeNodes($reviewer_id = 0) {

    $nids = [];
    if (is_numeric($reviewer_id) && !empty($reviewer_id)) {

      $database = \Drupal::database();
      $query = $database->select('mt_node_queue', 'nq');
      $query->fields('nq', array('nid'));
      $query->condition('reviewer_id', $reviewer_id , '=');
      $results = $query->execute();

      if (!empty($results)) {
        $nodes = $results->fetchAll();

        foreach ($nodes as $key => $node) {
          $nids[] = isset($node->nid) ? $node->nid : 0;
        }
      }
    }

    return $nids;
  }

  /**
   * To get node ids based on assigneee
   *
   * @param: reviewer_id: id of the assignee
   *
   * @return: array of node ids
   */
  function removeAssigneeNodes($reviewer_id = 0) {

    if (is_numeric($reviewer_id) && !empty($reviewer_id)) {
      $database = \Drupal::database();
      $database->delete('mt_node_queue')
        ->condition('reviewer_id', $reviewer_id)
        ->execute();
    }
  }

  /**
   * To get node ids based on queue
   *
   * @param: queue_id: id of the queue
   *
   * @return: array of node ids
   */
  function queueNodes($queue_id = 0) {

    $nids = [0];
    if (is_numeric($queue_id) && !empty($queue_id)) {

      $database = \Drupal::database();
      $query = $database->select('mt_queue', 'mq');
      $query->fields('mq', array('nid'));
      $query->condition('queue_id', $queue_id , '=');
      $results = $query->execute();

      if (!empty($results)) {
        $nodes = $results->fetchAll();

        foreach ($nodes as $key => $node) {
          $nids[] = isset($node->nid) ? $node->nid : 0;
        }
      }
    }
    return $nids;
  }

}